'use strict';

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _redux = require('redux');

var _reduxThunk = require('redux-thunk');

var _reduxThunk2 = _interopRequireDefault(_reduxThunk);

var _reactRouter = require('react-router');

var _history = require('history');

var _reactRouterRedux = require('react-router-redux');

var _reactIntl = require('react-intl');

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _defaultsDeep = require('lodash/defaultsDeep');

var _defaultsDeep2 = _interopRequireDefault(_defaultsDeep);

var _merge = require('lodash/merge');

var _merge2 = _interopRequireDefault(_merge);

var _scriptjs = require('scriptjs');

var _scriptjs2 = _interopRequireDefault(_scriptjs);

var _warning = require('warning');

var _warning2 = _interopRequireDefault(_warning);

var _cspace = require('./actions/cspace');

var _reducers = require('./reducers');

var _reducers2 = _interopRequireDefault(_reducers);

var _App = require('./components/App');

var _App2 = _interopRequireDefault(_App);

var _object = require('./plugins/record/object');

var _object2 = _interopRequireDefault(_object);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var loadPolyfills = function loadPolyfills(locale, callback) {
  if (window.Intl) {
    window.setTimeout(callback, 0);
  } else {
    var url = 'https://cdn.polyfill.io/v2/polyfill.min.js?features=Intl.~locale.' + locale;

    (0, _scriptjs2.default)(url, callback);
  }
}; /* global document, window */

var preparePlugins = function preparePlugins(plugins) {
  var preparedPlugins = {};

  var pluginContext = {
    Immutable: _immutable2.default,
    React: _react2.default,
    defineMessages: _reactIntl.defineMessages
  };

  Object.keys(plugins).forEach(function (type) {
    var pluginsForType = plugins[type];
    var preparedPluginsForType = {};

    Object.keys(pluginsForType).forEach(function (name) {
      var plugin = pluginsForType[name];
      var preparedPlugin = plugin(pluginContext);

      preparedPluginsForType[name] = preparedPlugin;
    });

    preparedPlugins[type] = preparedPluginsForType;
  });

  return preparedPlugins;
};

var defaultRecordPlugins = {
  object: (0, _object2.default)()
};

var defaultConfig = {
  basename: '',
  container: 'main',
  cspaceUrl: '',
  index: undefined,
  locale: 'en',
  messages: undefined,
  prettyUrls: false,
  plugins: {
    record: defaultRecordPlugins
  }
};

var resolveConfig = function resolveConfig(uiConfig) {
  if (typeof uiConfig === 'function') {
    return uiConfig((0, _merge2.default)({}, defaultConfig));
  }

  return (0, _defaultsDeep2.default)({}, uiConfig, defaultConfig);
};

module.exports = function (uiConfig) {
  var _resolveConfig = resolveConfig(uiConfig);

  var basename = _resolveConfig.basename;
  var container = _resolveConfig.container;
  var cspaceUrl = _resolveConfig.cspaceUrl;
  var index = _resolveConfig.index;
  var locale = _resolveConfig.locale;
  var messages = _resolveConfig.messages;
  var prettyUrls = _resolveConfig.prettyUrls;
  var plugins = _resolveConfig.plugins;


  var mountNode = document.querySelector(container);

  process.env.NODE_ENV !== 'production' ? (0, _warning2.default)(mountNode, 'No container element was found using the selector \'' + container + '\'. The CollectionSpace UI will not be rendered.') : void 0;

  if (mountNode) {
    (function () {
      process.env.NODE_ENV !== 'production' ? (0, _warning2.default)(mountNode !== document.body, 'The container element for the CollectionSpace UI found using the selector \'' + container + '\' is the document body. This may cause problems, and is not supported.') : void 0;

      var store = (0, _redux.createStore)(_reducers2.default, (0, _redux.applyMiddleware)(_reduxThunk2.default));
      var baseHistory = prettyUrls ? (0, _reactRouter.useRouterHistory)(_history.createHistory)({ basename: basename }) : _reactRouter.hashHistory;
      var history = (0, _reactRouterRedux.syncHistoryWithStore)(baseHistory, store);

      store.dispatch((0, _cspace.configureCSpace)({
        url: cspaceUrl
      }));

      var props = {
        history: history,
        index: index,
        locale: locale,
        messages: messages,
        store: store,
        plugins: preparePlugins(plugins)
      };

      loadPolyfills(locale, function () {
        (0, _reactDom.render)(_react2.default.createElement(_App2.default, props), mountNode);
      });
    })();
  }
};